import functools
import logging
from typing import NamedTuple
from .exceptions import UnhandledRecordData

logger = logging.getLogger(__name__)


# This is needed so that mypy is happy with our return annotations
class Record:  # type: ignore
    pass


def _generate_records():
    def _inner(func):
        @functools.wraps(func)
        def _fn(*args, **kwargs):
            headers, rows = func(*args, **kwargs)
            if all([isinstance(headers, tuple), isinstance(rows, list), headers, rows]):

                # headers variable contains the values listed here:
                # https://github.com/mkleehammer/pyodbc/wiki/Cursor#description
                annotations = [(header[0].replace(" ", "_") or "missing_column_name", header[1]) for header in headers]

                Record = NamedTuple("Record", annotations)  # type: ignore

                return [Record(*row) for row in rows]  # type: ignore # pylint: disable=not-callable

            else:
                unhandled_data_message = (
                    "Failed to generate object records. Either the returned rows were not a list "
                    "of tuples, or the query did not return any headers."
                )
                logger.error(f"{unhandled_data_message} See the received data below:")

                logger.error(f"Headers: {headers}")
                logger.error(f"Rows: {rows}")
                raise UnhandledRecordData(unhandled_data_message)

        return _fn

    return _inner
